@@ -4,7 +4,7 @@ import inspect
 from abc import ABC, abstractmethod
 from collections.abc import Callable
 from enum import Enum
-from typing import Any, List, Optional, Type, get_args, get_origin
+from typing import Any, List, Optional, Type, get_args, get_origin, get_type_hints
 
 from neuroglia.core.type_extensions import TypeExtensions
 
@@ -124,6 +124,54 @@ class ServiceProviderBase(ABC):
         """Creates a new service scope"""
         raise NotImplementedError()
 
+    def create_async_scope(self):
+        """
+        Creates an asynchronous service scope for resolving scoped services.
+
+        This method provides an async context manager that creates a service scope,
+        allowing scoped services to be properly resolved and automatically disposed
+        when the scope exits. This is essential for event-driven architectures where
+        handlers need access to scoped dependencies like repositories.
+
+        Returns:
+            An async context manager that yields a ServiceScope
+
+        Examples:
+            ```python
+            # Event processing with scoped services
+            async with service_provider.create_async_scope() as scope:
+                handler = scope.get_service(EventHandler)
+                await handler.handle_async(event)
+            # Scope automatically disposed here
+
+            # Multiple services in same scope
+            async with service_provider.create_async_scope() as scope:
+                repository = scope.get_service(Repository)
+                unit_of_work = scope.get_service(UnitOfWork)
+                await process_with_shared_context(repository, unit_of_work)
+            ```
+
+        See Also:
+            - Scoped Service Resolution: https://bvandewe.github.io/pyneuro/patterns/dependency-injection
+            - Event-Driven Architecture: https://bvandewe.github.io/pyneuro/features/simple-cqrs/
+        """
+        from contextlib import asynccontextmanager
+
+        @asynccontextmanager
+        async def _scope_context():
+            scope = self.create_scope()
+            try:
+                yield scope
+            finally:
+                # Dispose scope asynchronously if supported, otherwise synchronous
+                if hasattr(scope, "dispose_async"):
+                    await scope.dispose_async()
+                elif hasattr(scope, "dispose"):
+                    scope.dispose()
+
+        # Return the context manager itself (already callable)
+        return _scope_context()
+
 
 class ServiceScopeBase(ABC):
     """
@@ -275,17 +323,35 @@ class ServiceScope(ServiceScopeBase, ServiceProviderBase):
                 continue
             realized_services.append(self._build_service(descriptor))
 
-        # Only get singleton and transient services from root provider
-        # Scoped services should only come from the current scope
-        root_services = []
-        try:
-            # Call a special method that only returns singleton/transient services
-            root_services = self._root_service_provider._get_non_scoped_services(type)
-        except Exception:
-            # If there's an error getting root services, just use what we have from scope
-            pass
+        # Get singleton services from root provider
+        # IMPORTANT: Transient services must be built in THIS scope (not root)
+        # to allow them to resolve scoped dependencies correctly
+        root_singleton_descriptors = [descriptor for descriptor in self._all_service_descriptors if descriptor.service_type == type and descriptor.lifetime == ServiceLifetime.SINGLETON]
 
-        return realized_services + root_services
+        # Build transient services in THIS scope so they can access scoped dependencies
+        transient_descriptors = [descriptor for descriptor in self._all_service_descriptors if descriptor.service_type == type and descriptor.lifetime == ServiceLifetime.TRANSIENT]
+
+        # Get realized singletons from root provider (they're cached there)
+        root_services = []
+        for descriptor in root_singleton_descriptors:
+            try:
+                service = self._root_service_provider.get_service(descriptor.service_type)
+                if service is not None:
+                    root_services.append(service)
+            except Exception:
+                pass
+
+        # Build transient services in this scope
+        transient_services = []
+        for descriptor in transient_descriptors:
+            try:
+                service = self._build_service(descriptor)
+                transient_services.append(service)
+            except Exception as ex:
+                # If building fails, skip this service
+                pass
+
+        return realized_services + root_services + transient_services
 
     def _build_service(self, service_descriptor: ServiceDescriptor) -> any:
         """Builds a new scoped service"""
@@ -297,28 +363,49 @@ class ServiceScope(ServiceScopeBase, ServiceProviderBase):
             is_service_generic = not inspect.isclass(service_descriptor.implementation_type)
             service_generic_type = service_descriptor.implementation_type.__origin__ if is_service_generic else None
             service_type = service_descriptor.implementation_type if service_generic_type is None else service_generic_type
+
+            # Resolve string annotations (forward references) to actual types
+            try:
+                type_hints = get_type_hints(service_type.__init__)
+            except Exception:
+                # If get_type_hints fails, fall back to inspecting annotations directly
+                type_hints = {}
+
             service_init_args = [param for param in inspect.signature(service_type.__init__).parameters.values() if param.name not in ["self", "args", "kwargs"]]
             service_generic_args = TypeExtensions.get_generic_arguments(service_descriptor.implementation_type)
             service_args = dict[Type, any]()
             for init_arg in service_init_args:
+                # Get the resolved type hint (handles string annotations)
+                resolved_annotation = type_hints.get(init_arg.name, init_arg.annotation)
+
                 # Use typing.get_origin() and get_args() for robust generic type handling
-                origin = get_origin(init_arg.annotation)
-                args = get_args(init_arg.annotation)
+                origin = get_origin(resolved_annotation)
+                args = get_args(resolved_annotation)
 
                 # Determine the dependency type to resolve
                 if origin is not None and args:
                     # It's a parameterized generic type (e.g., Repository[User, int])
-                    # Use the annotation directly - it's already properly parameterized
-                    # The DI container will match it against registered types
-                    # Note: TypeVar substitution is handled by get_generic_arguments() at service level
-                    dependency_type = init_arg.annotation
+                    # Check if it contains type variables that need substitution
+                    # (e.g., CacheRepositoryOptions[TEntity, TKey] -> CacheRepositoryOptions[MozartSession, str])
+                    dependency_type = TypeExtensions._substitute_generic_arguments(resolved_annotation, service_generic_args)
                 else:
-                    # Simple non-generic type
-                    dependency_type = init_arg.annotation
+                    # Simple non-generic type (use resolved annotation, not raw annotation)
+                    dependency_type = resolved_annotation
 
                 dependency = self.get_service(dependency_type)
                 if dependency is None and init_arg.default == init_arg.empty and init_arg.name != "self":
-                    raise Exception(f"Failed to build service of type '{service_descriptor.service_type.__name__}' because the service provider failed to resolve service '{dependency_type.__name__}'")
+                    # Safe error message generation - handle all annotation types:
+                    # 1. String annotations (forward references): "ClassName"
+                    # 2. Types without __name__: typing constructs like Union, Optional
+                    # 3. Regular types with __name__: normal classes
+                    def _get_type_name(t) -> str:
+                        if isinstance(t, str):
+                            return t  # Already a string (forward reference)
+                        return getattr(t, "__name__", str(t))
+
+                    service_type_name = _get_type_name(service_descriptor.service_type)
+                    dependency_type_name = _get_type_name(dependency_type)
+                    raise Exception(f"Failed to build service of type '{service_type_name}' because the service provider failed to resolve service '{dependency_type_name}'")
                 service_args[init_arg.name] = dependency
             service = service_descriptor.implementation_type(**service_args)
 
@@ -340,19 +427,31 @@ class ServiceScope(ServiceScopeBase, ServiceProviderBase):
                     pass
         self._realized_scoped_services = dict[Type, List]()
 
-    def create_scope(self) -> ServiceScopeBase:
-        return self
-
-    def dispose(self):
+    async def dispose_async(self):
+        """Asynchronously dispose of scoped services"""
         for services in self._realized_scoped_services.values():
             for service in services:
                 try:
-                    if hasattr(service, "__exit__"):
+                    # Try async context manager exit first
+                    if hasattr(service, "__aexit__"):
+                        await service.__aexit__(None, None, None)
+                    # Fall back to sync context manager exit
+                    elif hasattr(service, "__exit__"):
                         service.__exit__(None, None, None)
+
+                    # Also call dispose() method if it exists (for explicit resource cleanup)
+                    if hasattr(service, "dispose"):
+                        result = service.dispose()
+                        # If dispose() returns a coroutine, await it
+                        if hasattr(result, "__await__"):
+                            await result
                 except:
                     pass
         self._realized_scoped_services = dict[Type, List]()
 
+    def create_scope(self) -> ServiceScopeBase:
+        return self
+
 
 class ServiceProvider(ServiceProviderBase):
     """Represents the default implementation of the IServiceProvider class"""
@@ -470,28 +569,49 @@ class ServiceProvider(ServiceProviderBase):
             is_service_generic = not inspect.isclass(service_descriptor.implementation_type)  # if implementation_type is not a class, then it must be a generic type
             service_generic_type = service_descriptor.implementation_type.__origin__ if is_service_generic else None  # retrieve the generic type, used to determine the __init__ args
             service_type = service_descriptor.implementation_type if service_generic_type is None else service_generic_type  # get the type used to determine the __init__ args: the implementation type as is or its generic type definition
+
+            # Resolve string annotations (forward references) to actual types
+            try:
+                type_hints = get_type_hints(service_type.__init__)
+            except Exception:
+                # If get_type_hints fails, fall back to inspecting annotations directly
+                type_hints = {}
+
             service_init_args = [param for param in inspect.signature(service_type.__init__).parameters.values() if param.name not in ["self", "args", "kwargs"]]  # gets the __init__ args and leave out self, args and kwargs
             service_generic_args = TypeExtensions.get_generic_arguments(service_descriptor.implementation_type)  # gets the generic args: we will need them to substitute the type args of potential generic dependencies
             service_args = dict[Type, any]()
             for init_arg in service_init_args:
+                # Get the resolved type hint (handles string annotations)
+                resolved_annotation = type_hints.get(init_arg.name, init_arg.annotation)
+
                 # Use typing.get_origin() and get_args() for robust generic type handling
-                origin = get_origin(init_arg.annotation)
-                args = get_args(init_arg.annotation)
+                origin = get_origin(resolved_annotation)
+                args = get_args(resolved_annotation)
 
                 # Determine the dependency type to resolve
                 if origin is not None and args:
                     # It's a parameterized generic type (e.g., Repository[User, int])
-                    # Use the annotation directly - it's already properly parameterized
-                    # The DI container will match it against registered types
-                    # Note: TypeVar substitution is handled by get_generic_arguments() at service level
-                    dependency_type = init_arg.annotation
+                    # Check if it contains type variables that need substitution
+                    # (e.g., CacheRepositoryOptions[TEntity, TKey] -> CacheRepositoryOptions[MozartSession, str])
+                    dependency_type = TypeExtensions._substitute_generic_arguments(resolved_annotation, service_generic_args)
                 else:
-                    # Simple non-generic type
-                    dependency_type = init_arg.annotation
+                    # Simple non-generic type (use resolved annotation, not raw annotation)
+                    dependency_type = resolved_annotation
 
                 dependency = self.get_service(dependency_type)
                 if dependency is None and init_arg.default == init_arg.empty and init_arg.name != "self":
-                    raise Exception(f"Failed to build service of type '{service_descriptor.service_type.__name__}' because the service provider failed to resolve service '{dependency_type.__name__}'")
+                    # Safe error message generation - handle all annotation types:
+                    # 1. String annotations (forward references): "ClassName"
+                    # 2. Types without __name__: typing constructs like Union, Optional
+                    # 3. Regular types with __name__: normal classes
+                    def _get_type_name(t) -> str:
+                        if isinstance(t, str):
+                            return t  # Already a string (forward reference)
+                        return getattr(t, "__name__", str(t))
+
+                    service_type_name = _get_type_name(service_descriptor.service_type)
+                    dependency_type_name = _get_type_name(dependency_type)
+                    raise Exception(f"Failed to build service of type '{service_type_name}' because the service provider failed to resolve service '{dependency_type_name}'")
                 service_args[init_arg.name] = dependency
             service = service_descriptor.implementation_type(**service_args)
         if service_descriptor.lifetime != ServiceLifetime.TRANSIENT:
